/*
** Module   : recid.java
** Abstract : Progress 4GL compatible record ID
**
** Copyright (c) 2005-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- -------------------------Description------------------------------
** 001 SVL 20090225   @41371 Created initial version. Progress 4GL compatible
**                           record ID data type.
** 002 ECF 20090521   @42419 Added duplicate() method.
** 003 CA  20130121          Added missing c'tor.
**                           Added c'tor which builds a new instance from a
**                           BaseDataType value, if possible. This should be
**                           used only when constructing an instance using the
**                           value returned by a DYNAMIC-FUNCTION call; 4GL
**                           does some special casting in this case, which is
**                           not implemented yet.
** 004 GES 20130322          Fixed the BDT c'tor.
** 005 CA  20131001          Eliminated custom group/dec separators when format is used. In this
**                           case, they must default to comma (,) for group and dot (.) for
**                           decimal separator.
*/

package com.goldencode.p2j.util;

/**
 * A class that represents a Progress 4GL compatible record ID. It is the same
 * as {@link integer} except default format.
 */
public class recid
extends integer
{
   /** Default format string for this data type. */
   private static String defFmt = ">>>>>>9";

   /**
    * Default constructor which creates an instance representing the
    * <code>unknown value</code>.
    */
   public recid()
   {
      super();
   }

   /**
    * This is a special c'tor which should be used only when converting the
    * value returned by a function or method with polymorphic return type into the
    * expected type (i.e. DYNAMIC-FUNCTION()).  In such cases, the 4GL does some
    * automatic type conversion (see {@link #assign(BaseDataType,boolean)}).
    * 
    * @param    value
    *           The value to be used for this instance.
    */
   public recid(BaseDataType value)
   {
      if (value == null || value.isUnknown())
      {
         setUnknown();
      }
      else
      {
         assign(value, false);
      }
   }

   /**
    * Constructs an instance after casting the parameter into the internal
    * representation of this class.  If the value is larger than
    * <code>Integer.MAX_VALUE</code> or smaller than
    * <code>Integer.MIN_VALUE</code> then some loss of magnitude will
    * occur (the resulting value will be set to the maximum or minimum
    * value respectively).
    * <p>
    * It is not possible to yield an <code>unknown value</code> from this
    * constructor without calling the method {@link #setUnknown} separate.
    *
    * @param   value
    *          The value to be used for this instance.
    */
   public recid(long value)
   {
      super(value);
   }

   /**
    * Constructs an instance after casting the parameter into the internal
    * representation of this class.  If the value is larger than
    * <code>Integer.MAX_VALUE</code> or smaller than
    * <code>Integer.MIN_VALUE</code> then some loss of magnitude will
    * occur (the resulting value will be set to the maximum or minimum
    * value respectively).
    * <p>
    * If the parameter's represents the <code>unknown value</code>, this
    * instance will also represent the <code>unknown value</code>.
    *
    * @param   value
    *          The value to be used for this instance.
    */
   public recid(integer value)
   {
      super(value);
   }

   /**
    * Constructs an instance after copying the parameter's data into the
    * internal representation of this class.  No loss of precision will
    * occur.
    * <p>
    * If the parameter's represents the <code>unknown value</code>, this
    * instance will also represent the <code>unknown value</code>.
    *
    * @param   value
    *          The value to be used for this instance.
    */
   public recid(recid value)
   {
      super(value);
   }

   /**
    * Constructs an instance after converting the string representation
    * of a number into an integer.  Due to rounding, a loss of precision
    * may occur in this conversion (all digits to the right of the decimal
    * point will be truncated).  In addition, if the number is larger than
    * can be stored in a Java <code>int</code>, loss of magnitude will
    * occur.
    *
    * @param   value
    *          The value to be used for this instance.
    */
   public recid(String value)
   {
      super(value);
   }

   /**
    * Does the same as standard <code>clone()</code> method but returns an
    * instance of <code>BaseDataType</code> and doesn't throw the
    * <code>CloneNotSupportedException</code>.
    *
    * @return   A clone of this instance.  
    */
   public BaseDataType duplicate()
   {
      return new recid(this);
   }
   
   /**
    * Returns the default format string for this given data type.
    *
    * @return   The default format string for this data.
    */
   public String obtainDefaultFormat()
   {
      return defFmt;
   }

   /**
    * Dynamically builds the default format string for this given data type.
    *
    * @return   The default format string for this data type.
    */
   public String buildDefaultFormat()
   {
      return defFmt;
   }
}
